@@ -1,5 +1,192 @@
-# handy
-sock <- safely(socketConnection)
+# the documented, memoised versions of these are at the bottom
+
+.bulk_origin <- function(ips, timeout=getOption("timeout")) {
+
+  host <- "v4.whois.cymru.com"
+  port <- 43
+
+  # setup query
+  cmd <- "begin\nverbose\n"
+  ips_c <- paste(unlist(ips), collapse="\n")
+  cmd <- sprintf("%s%s\nend\n", cmd, ips_c)
+
+  # setup connection and post query
+  con <- sock(host=host, port=port, blocking=TRUE, open="r+", timeout=timeout)
+  if (is.null(con$result)) {
+    message("Error opening connection to v4.whois.cymru.com")
+    data.frame(
+      as = rep(NA, length(ips)),
+      ip = rep(NA, length(ips)),
+      bgp_prefix = rep(NA, length(ips)),
+      cc = rep(NA, length(ips)),
+      registry = rep(NA, length(ips)),
+      allocated = rep(NA, length(ips)),
+      as_name = rep(NA, length(ips))
+    ) -> out
+    class(out) <- c("tbl_df", "tbl", "data.frame")
+    return(out)
+  }
+
+  con <- con$result
+  cat(cmd, file=con)
+  response <- readLines(con)
+  close(con)
+
+  if (length(response) == 0) {
+    message("Error reading from connection to v4.whois.cymru.com")
+    data.frame(
+      as = rep(NA, length(ips)),
+      ip = rep(NA, length(ips)),
+      bgp_prefix = rep(NA, length(ips)),
+      cc = rep(NA, length(ips)),
+      registry = rep(NA, length(ips)),
+      allocated = rep(NA, length(ips)),
+      as_name = rep(NA, length(ips))
+    ) -> out
+    class(out) <- c("tbl_df", "tbl", "data.frame")
+    return(out)
+  }
+
+  # trim header, split fields and convert results
+  trim_df(
+    read.csv(
+      textConnection(tail(response, -1)),
+      stringsAsFactors = FALSE, 
+      sep = "|", 
+      header = FALSE
+    )
+  ) -> response
+  names(response) <- c("as", "ip", "bgp_prefix", "cc","registry", "allocated", "as_name")
+  response[(response == "NA")] <- NA
+  class(response) <- c("tbl_df", "tbl", "data.frame")
+  return(response)
+
+}
+
+.bulk_peer <- function(ips, timeout=getOption("timeout")) {
+
+  host <- "peer.whois.cymru.com"
+  # host <- "v4-peer.whois.cymru.com"
+  port <- 43
+
+  # setup query
+  cmd <- "begin\nverbose\n"
+  ips_c <- paste(unlist(ips), collapse="\n")
+  cmd <- sprintf("%s%s\nend\n", cmd, ips_c)
+
+  # setup connection and post query
+  con <- sock(host=host, port=port, blocking=TRUE, open="r+", timeout=timeout)
+  if (is.null(con$result)) {
+    message("Error opening connection to v4-peer.whois.cymru.com")
+    data.frame(
+      peer_as = rep(NA, length(ips)),
+      ip = rep(NA, length(ips)),
+      bgp_prefix = rep(NA, length(ips)),
+      cc = rep(NA, length(ips)),
+      registry = rep(NA, length(ips)),
+      allocated = rep(NA, length(ips)),
+      peer_as_name = rep(NA, length(ips))
+    ) -> out
+    class(out) <- c("tbl_df", "tbl", "data.frame")
+    return(out)
+  }
+
+  con <- con$result
+  cat(cmd, file=con)
+  response <- readLines(con)
+  close(con)
+
+  if (length(response) == 0) {
+    message("Error reading from connection to v4-peer.whois.cymru.com")
+    data.frame(
+      peer_as = rep(NA, length(ips)),
+      ip = rep(NA, length(ips)),
+      bgp_prefix = rep(NA, length(ips)),
+      cc = rep(NA, length(ips)),
+      registry = rep(NA, length(ips)),
+      allocated = rep(NA, length(ips)),
+      peer_as_name = rep(NA, length(ips))
+    ) -> out
+    class(out) <- c("tbl_df", "tbl", "data.frame")
+    return(out)
+  }
+
+
+  # trim header, split fields and convert results
+  trim_df(
+    read.csv(
+      textConnection(tail(response, -1)),
+      stringsAsFactors = FALSE, 
+      sep = "|", 
+      header = FALSE
+    )
+  ) -> response
+  names(response) <- c("peer_as", "ip", "bgp_prefix", "cc", "registry", "allocated", "peer_as_name")
+  response[(response == "NA")] <- NA
+  class(response) <- c("tbl_df", "tbl", "data.frame")
+  return(response)
+
+}
+
+.bulk_origin_asn <- function(asns, timeout=getOption("timeout")) {
+
+  host <- "v4.whois.cymru.com"
+  port <- 43
+
+  # setup query
+  cmd <- "begin\nverbose\n"
+  ips <- paste(unlist(ifelse(grepl("^AS", asns), asns,
+                             sprintf("AS%s", asns))), collapse="\n")
+  cmd <- sprintf("%s%s\nend\n", cmd, ips)
+
+  # setup connection and post query
+  con <- sock(host=host, port=port, blocking=TRUE, open="r+", timeout=timeout)
+  if (is.null(con$result)) {
+    message("Error opening connection to v4.whois.cymru.com")
+    data.frame(
+      as = rep(NA, length(asns)),
+      cc = rep(NA, length(asns)),
+      registry = rep(NA, length(asns)),
+      allocated = rep(NA, length(asns)),
+      as_name = rep(NA, length(asns))
+    ) -> out
+    class(out) <- c("tbl_df", "tbl", "data.frame")
+    return(out)
+  }
+
+  con <- con$result
+  cat(cmd, file=con)
+  response <- readLines(con)
+  close(con)
+
+  if (length(response) == 0) {
+    message("Error reading from connection to v4.whois.cymru.com")
+    data.frame(
+      as = rep(NA, length(asns)),
+      cc = rep(NA, length(asns)),
+      registry = rep(NA, length(asns)),
+      allocated = rep(NA, length(asns)),
+      as_name = rep(NA, length(asns))
+    ) -> out
+    class(out) <- c("tbl_df", "tbl", "data.frame")
+    return(out)
+  }
+
+  # trim header, split fields and convert results
+  trim_df(
+    read.csv(
+      textConnection(tail(response, -1)),
+      stringsAsFactors = FALSE, 
+      sep = "|", 
+      header = FALSE
+    )
+  ) -> response
+  names(response) <- c("as", "cc", "registry", "allocated", "as_name")
+  response[(response == "NA")] <- NA
+  class(response) <- c("tbl_df", "tbl", "data.frame")
+  return(response)
+
+}
 
 #' Retrieves BGP Origin ASN info for a list of IPv4 addresses
 #'
@@ -30,55 +217,8 @@ sock <- safely(socketConnection)
 #' @examples \dontrun{
 #' bulk_origin(c("68.22.187.5", "207.229.165.18", "198.6.1.65"))
 #' }
-bulk_origin <- function(ips, timeout=getOption("timeout")) {
+bulk_origin <- memoise::memoise(.bulk_origin)
 
-  host <- "v4.whois.cymru.com"
-  port <- 43
-
-  # setup query
-  cmd <- "begin\nverbose\n"
-  ips_c <- paste(unlist(ips), collapse="\n")
-  cmd <- sprintf("%s%s\nend\n", cmd, ips_c)
-
-  # setup connection and post query
-  con <- sock(host=host, port=port, blocking=TRUE, open="r+", timeout=timeout)
-  if (is.null(con$result)) {
-    message("Error opening connection to v4.whois.cymru.com")
-    return(data.frame(as=rep(NA, length(ips)),
-                      ip=rep(NA, length(ips)),
-                      bgp_prefix=rep(NA, length(ips)),
-                      cc=rep(NA, length(ips)),
-                      registry=rep(NA, length(ips)),
-                      allocated=rep(NA, length(ips)),
-                      as_name=rep(NA, length(ips))))
-  }
-
-  con <- con$result
-  cat(cmd, file=con)
-  response <- readLines(con)
-  close(con)
-
-  if (length(response) == 0) {
-    message("Error reading from connection to v4.whois.cymru.com")
-    return(data.frame(as=rep(NA, length(ips)),
-                      ip=rep(NA, length(ips)),
-                      bgp_prefix=rep(NA, length(ips)),
-                      cc=rep(NA, length(ips)),
-                      registry=rep(NA, length(ips)),
-                      allocated=rep(NA, length(ips)),
-                      as_name=rep(NA, length(ips))))
-  }
-
-  # trim header, split fields and convert results
-  response <- trim_df(read.csv(textConnection(tail(response, -1)),
-                               stringsAsFactors=FALSE, sep="|", header=FALSE))
-  names(response) <- c("as", "ip", "bgp_prefix", "cc",
-                       "registry", "allocated", "as_name")
-  response[response=="NA"] <- NA
-
-  return(response)
-
-}
 
 #' Retrieves BGP Peer ASN info for a list of IPv4 addresses
 #'
@@ -109,56 +249,8 @@ bulk_origin <- function(ips, timeout=getOption("timeout")) {
 #' @examples \dontrun{
 #' bulk_peer(c("68.22.187.5", "207.229.165.18", "198.6.1.65"))
 #' }
-bulk_peer <- function(ips, timeout=getOption("timeout")) {
+bulk_peer <- memoise::memoise(.bulk_peer)
 
-  host <- "v4-peer.whois.cymru.com"
-  port <- 43
-
-  # setup query
-  cmd <- "begin\nverbose\n"
-  ips_c <- paste(unlist(ips), collapse="\n")
-  cmd <- sprintf("%s%s\nend\n", cmd, ips_c)
-
-  # setup connection and post query
-  con <- sock(host=host, port=port, blocking=TRUE, open="r+", timeout=timeout)
-  if (is.null(con$result)) {
-    message("Error opening connection to v4-peer.whois.cymru.com")
-    return(data.frame(peer_as=rep(NA, length(ips)),
-                      ip=rep(NA, length(ips)),
-                      bgp_prefix=rep(NA, length(ips)),
-                      cc=rep(NA, length(ips)),
-                      registry=rep(NA, length(ips)),
-                      allocated=rep(NA, length(ips)),
-                      peer_as_name=rep(NA, length(ips))))
-  }
-
-  con <- con$result
-  cat(cmd, file=con)
-  response <- readLines(con)
-  close(con)
-
-  if (length(response) == 0) {
-    message("Error reading from connection to v4-peer.whois.cymru.com")
-    return(data.frame(peer_as=rep(NA, length(ips)),
-                      ip=rep(NA, length(ips)),
-                      bgp_prefix=rep(NA, length(ips)),
-                      cc=rep(NA, length(ips)),
-                      registry=rep(NA, length(ips)),
-                      allocated=rep(NA, length(ips)),
-                      peer_as_name=rep(NA, length(ips))))
-  }
-
-
-  # trim header, split fields and convert results
-  response <- trim_df(read.csv(textConnection(tail(response, -1)),
-                               stringsAsFactors=FALSE, sep="|", header=FALSE))
-  names(response) <- c("peer_as", "ip", "bgp_prefix", "cc",
-                       "registry", "allocated", "peer_as_name")
-  response[response=="NA"] <- NA
-
-  return(response)
-
-}
 
 #' Retrieves BGP Origin ASN info for a list of ASN ids
 #'
@@ -188,48 +280,4 @@ bulk_peer <- function(ips, timeout=getOption("timeout")) {
 #' bulk_origin_asn(c(22822, 1273, 2381, 2603, 2914, 3257, 3356, 11164,
 #'                   174, 286, 1299, 2914, 3257, 3356, 3549, 22822))
 #' }
-bulk_origin_asn <- function(asns, timeout=getOption("timeout")) {
-
-  host <- "v4.whois.cymru.com"
-  port <- 43
-
-  # setup query
-  cmd <- "begin\nverbose\n"
-  ips <- paste(unlist(ifelse(grepl("^AS", asns), asns,
-                             sprintf("AS%s", asns))), collapse="\n")
-  cmd <- sprintf("%s%s\nend\n", cmd, ips)
-
-  # setup connection and post query
-  con <- sock(host=host, port=port, blocking=TRUE, open="r+", timeout=timeout)
-  if (is.null(con$result)) {
-    message("Error opening connection to v4.whois.cymru.com")
-    return(data.frame(as=rep(NA, length(asns)),
-                      cc=rep(NA, length(asns)),
-                      registry=rep(NA, length(asns)),
-                      allocated=rep(NA, length(asns)),
-                      as_name=rep(NA, length(asns))))
-  }
-
-  con <- con$result
-  cat(cmd, file=con)
-  response <- readLines(con)
-  close(con)
-
-  if (length(response) == 0) {
-    message("Error reading from connection to v4.whois.cymru.com")
-    return(data.frame(as=rep(NA, length(asns)),
-                      cc=rep(NA, length(asns)),
-                      registry=rep(NA, length(asns)),
-                      allocated=rep(NA, length(asns)),
-                      as_name=rep(NA, length(asns))))
-  }
-
-  # trim header, split fields and convert results
-  response <- trim_df(read.csv(textConnection(tail(response, -1)),
-                               stringsAsFactors=FALSE, sep="|", header=FALSE))
-  names(response) <- c("as", "cc", "registry", "allocated", "as_name")
-  response[response=="NA"] <- NA
-
-  return(response)
-
-}
+bulk_origin_asn <- memoise::memoise(.bulk_origin_asn)
